#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "ical.h"

void
dump(Param *root)
{
	Attr *a;
	Value *v;
	Param *p;

	for(p = root; p; p = p->next){
		print("%s\n", p->str);

		for(a = p->attrs; a; a = a->next)
			print("	%s=%s\n", a->name, a->value);

		for(v = p->values; v; v = v->next){
			if(v == p->values)
				print("\t\t%s\n", v->str);
			else
				print("\n\n%s\n", v->str);
		}
	}
}

static void
usage(void)
{
	fprint(2, "usage: %s [-d] [file.ical]\n", argv0);
}

void
main(int argc, char *argv[])
{
	int i, debug;
	Param *root;
	Biobuf *bp, out, inp;

	debug = 0;
	ARGBEGIN{
	case 'd':
		debug++;
		break;
	default:
		usage();
		break;
	}ARGEND;
	
	Binit(&out, OWRITE, 1);
	if(argc == 0){
		Binit(&inp, OREAD, 0);
		root = ical_parse(&inp, "stdin");
		calendar(&out, root);
		ical_free(root);
	}
	else
	for(i = 0; i < argc; i++){
		if((bp = Bopen(argv[i], OREAD)) == nil){
			fprint(2, "%s: cannot open, %r\n", argv0);
			continue;
		}
		root = ical_parse(bp, argv[i]);
		if(debug)
			dump(root);
		calendar(&out, root);
		ical_free(root);
		Bterm(bp);
	}
	exits(nil);
}
