#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "ical.h"

enum  {
	Sparam,
	Sattr,
	Svalue
};

static int Line;
static char *File;

static char *
dequote(char *s)
{
	char *t;

	if(*s != '"')
		return s;
	if((t = strrchr(s, '"')) == nil)
		return s;
	if(strcmp(t, "\"") != 0)
		return s;
	*t = 0;
	return s+1;
}

static Param *
split(Param *root, char *buf)
{
	int esc;
	char *s, *e, *t;
	Attr *a, **nattr;
	Rune S, E, quote;
	int n, len, state;
	Param *p, **nparam;
	Value *v, **nvalue;

	esc = 0;
	quote = 0;
	state = Sparam;
	s = e = buf;

	nparam = &root;
	nattr = nil;
	nvalue = nil;
	for(p = root; p; p = p->next)
		nparam = &p->next; 

	n = 0;
	do{
		e += n;
		chartorune(&S, e);
		n = chartorune(&E, e);
		len = (e-s)+1;
		switch(state){
		case Sparam:
			if(E == L';' || E == L':'){
				p = mallocz(sizeof(Param) + len, 1);
				p->str = (char *)&p[1];
				strecpy(p->str, p->str+len, s);

				*nparam = p;
				nparam = &p->next;
				nattr = &p->attrs;
				nvalue = &p->values;

				if(E == L';')
					state = Sattr;
				else
					state = Svalue;
				s = e+n;
			}
			break;
		case Sattr:
			if(E == L'"'){
				if(!quote)
					quote = S;
				else
				if(S == quote)
					quote = 0;
				break;
			}
			if(quote)
				break;
			if(E == L';' || E == L':' || E == 0){
				a = mallocz(sizeof(Attr) + len, 1);
				a->name = (char *)&a[1];
				strecpy(a->name, a->name+len, s);
				if((t = strchr(a->name, '=')) != nil){
					*t = 0;
					a->value = t+1;
					a->value = dequote(t+1);
				}
				else{
					fprint(2, "%s: parser error, %s contains no '='\n", argv0, a->name);
					a->value = "";
				}

				*nattr = a;
				nattr = &a->next;

				if(E == L':')
					state = Svalue;
				s = e+n;
				break;
			}
			break;
		case Svalue:
			if(E == L'\\'){
				esc = 1;
				break;
			}

			if(E == 0 || (E == L',' && !esc)){
				v = mallocz(sizeof(Value) + len, 1);
				v->str = (char *)&v[1];
				strecpy(v->str, v->str+len, s);
				*nvalue = v;
				nvalue = &v->next;

				if(E == L'\n')
					state = Sparam;
				s = e+n;
			}
			esc = 0;
			break;
		}
	}while(*e);

	if(e +n != s){
		fprint(2, "%s: %s:%d junk at EOS, line='%s' junk='%s'\n", argv0, File, Line, buf, e+n);
		exits("bad");
	}
	return root;
}

Param *
ical_parse(Biobuf *bp, char *file)
{
	Param *root;
	char *str, *buf;
	int c, got, alloc, pos;

	c = 0;
	root = nil;
	Line = 0;
	File = file;
	alloc = 0;
	pos = 0;
	buf = nil;
	while((str = Brdline(bp, '\n')) != nil && c != -1){
		got = Blinelen(bp);
		Line++;

		if(pos+got >= alloc){
			alloc += got;
			buf = realloc(buf, alloc);
			assert(buf != nil);
		}

		memcpy(buf+pos, str, got);

		while(isspace(str[got-1]))			/* trailing whitespace */
			got--;
		pos += got;
		buf[pos] = 0;

		c = Bgetc(bp);
		if(isspace(c)){						/* is a continuation str? */
			do{
				c = Bgetc(bp);
			}while(isspace(c));
			Bungetc(bp);
			continue;
		}
		Bungetc(bp);
		root = split(root, buf);
		pos = 0;
	}

	free(buf);
	return root;
}

void
ical_free(Param *root)
{
	Attr *a, *na;
	Value *v, *nv;
	Param *p, *np;

	for(p = root; p; p = np){
		np = p->next;
		for(a = p->attrs; a; a = na){
			na = a->next;
			free(a);
		}
		for(v = p->values; v; v = nv){
			nv = v->next;
			free(v);
		}			
		free(p);
	}
}

